package com.diagbot.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.diagbot.dto.AverageStatisticsDTO;
import com.diagbot.dto.DeptEntryNumDTO;
import com.diagbot.dto.DeptNumDTO;
import com.diagbot.dto.EntryNumDTO;
import com.diagbot.dto.EntryNumGroupDTO;
import com.diagbot.dto.NumDTO;
import com.diagbot.dto.QcResultPercentDTO;
import com.diagbot.entity.QcresultInfo;
import com.diagbot.mapper.QcresultInfoMapper;
import com.diagbot.service.QcresultInfoService;
import com.diagbot.vo.FilterPageByDeptVO;
import com.diagbot.vo.FilterPageVO;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.QcresultFilterVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 质控评分结果信息
每次评分增加一条信息，前面所有评分is_deleted全部设置为Y 服务实现类
 * </p>
 *
 * @author zhoutg
 * @since 2020-04-13
 */
@Service
public class QcresultInfoServiceImpl extends ServiceImpl<QcresultInfoMapper, QcresultInfo> implements QcresultInfoService {

    /**
     * 筛选质控记录
     *
     * @param qcresultFilterVO
     * @return
     */
    @Override
    public int getQcresultSelectively(QcresultFilterVO qcresultFilterVO) {
        return baseMapper.getQcresultSelectively(qcresultFilterVO);
    }

    /**
     * 质控病历数统计
     *
     * @param qcresultFilterVO
     * @return
     */
    @Override
    public int resultCount(QcresultFilterVO qcresultFilterVO) {
        return baseMapper.resultCount(qcresultFilterVO);
    }


    /**
     * 按科室统计各等级病历百分比
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<QcResultPercentDTO> levelPercentGroupByDept(FilterVO filterVO) {
        return baseMapper.levelPercentGroupByDept(filterVO);
    }

    /**
     * 按模块统计缺陷数
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<NumDTO> entryCountGroupByCase(FilterVO filterVO) {
        return baseMapper.entryCountGroupByCase(filterVO);
    }

    /**
     * 条目缺陷分组统计
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<NumDTO> entryCountGroupByEntry(FilterVO filterVO) {
        return baseMapper.entryCountGroupByEntry(filterVO);
    }

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<AverageStatisticsDTO> getAverageScore(FilterVO filterVO) {
        return baseMapper.getAverageScore(filterVO);
    }

    /**
     * 按科室统计质控病历数
     *
     * @param qcresultFilterVO
     * @return
     */
    @Override
    public List<NumDTO> resultCountByDept(QcresultFilterVO qcresultFilterVO) {
        return baseMapper.resultCountByDept(qcresultFilterVO);
    }

    /**
     * 各模块缺陷占比-按科室
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<DeptNumDTO> entryCountGroupByCaseAndDept(FilterVO filterVO) {
        return baseMapper.entryCountGroupByCaseAndDept(filterVO);
    }

    /**
     * 条目缺陷占比-按科室
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<DeptNumDTO> entryCountGroupByEntryAndDept(FilterVO filterVO) {
        return baseMapper.entryCountGroupByEntryAndDept(filterVO);
    }

    /**
     * 各模块缺陷占比（分页）
     *
     * @param filterPageVO
     * @return
     */
    @Override
    public IPage<NumDTO> entryCountGroupByCasePage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        return baseMapper.entryCountGroupByCasePage(filterPageVO);
    }

    /**
     * 缺陷详情（分页）
     *
     * @param filterPageVO
     * @return
     */
    @Override
    public IPage<EntryNumGroupDTO> entryCountGroupByEntryPage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        return baseMapper.entryCountGroupByEntryPage(filterPageVO);
    }

    /**
     * 各科室质控平均分（分页）
     *
     * @param filterPageVO
     * @return
     */
    @Override
    public IPage<AverageStatisticsDTO> getAverageScoreByDeptPage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        return baseMapper.getAverageScoreByDeptPage(filterPageVO);
    }

    /**
     * 各科室甲级病历占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    @Override
    public IPage<QcResultPercentDTO> levelPercentGroupByDeptPage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        return baseMapper.levelPercentGroupByDeptPage(filterPageVO);
    }

    /**
     * 各模块缺陷占比-科室（分页）
     *
     * @param filterPageByDeptVO
     * @return
     */
    @Override
    public IPage<DeptNumDTO> entryCountGroupByCaseAndDeptPage(@Param("filterPageByDeptVO") FilterPageByDeptVO filterPageByDeptVO) {
        return baseMapper.entryCountGroupByCaseAndDeptPage(filterPageByDeptVO);
    }

    /**
     * 缺陷详情-科室（分页）
     *
     * @param filterPageByDeptVO
     * @return
     */
    @Override
    public IPage<DeptEntryNumDTO> entryCountGroupByEntryAndDeptPage(@Param("filterPageByDeptVO") FilterPageByDeptVO filterPageByDeptVO) {
        return baseMapper.entryCountGroupByEntryAndDeptPage(filterPageByDeptVO);
    }

    /**
     * 条目缺陷占比（内页）
     *
     * @return
     */
    @Override
    public IPage<EntryNumDTO> entryGroupByEntryInnerPage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        return baseMapper.entryGroupByEntryInnerPage(filterPageVO);
    }

    /**
     * 条目缺陷占比-科室（内页）
     *
     * @param filterPageByDeptVO
     * @return
     */
    @Override
    public IPage<DeptEntryNumDTO> entryGroupByEntryAndDeptInnerPage(@Param("filterPageByDeptVO") FilterPageByDeptVO filterPageByDeptVO) {
        return baseMapper.entryGroupByEntryAndDeptInnerPage(filterPageByDeptVO);
    }

    /**
     * 各科室质控平均分（首页）-根据内外科系统统计
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<AverageStatisticsDTO> getAverageScoreByDeptClass(FilterVO filterVO) {
        return baseMapper.getAverageScoreByDeptClass(filterVO);
    }

    /**
     * 单项否决缺陷占比
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<EntryNumDTO> entryRejectPercent(FilterVO filterVO) {
        return baseMapper.entryRejectPercent(filterVO);
    }

    /**
     * 各科室甲/乙/丙级病历占比
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<DeptNumDTO> qcResultLevelPercent(FilterVO filterVO) {
        return baseMapper.qcResultLevelPercent(filterVO);
    }
}